@@ -1,10 +1,10 @@
-use russell_lab::*;
-use russell_sparse::*;
+use russell_lab::Vector;
+use russell_sparse::{ConfigSolver, LinSolKind, Solver, SparseTriplet, Symmetry};
 
-fn test_solver(name: LinSol) {
+fn test_solver(name: LinSolKind) {
     match name {
-        LinSol::Mmp => println!("Testing MMP solver\n"),
-        LinSol::Umf => println!("Testing UMF solver\n"),
+        LinSolKind::Mmp => println!("Testing MMP solver\n"),
+        LinSolKind::Umf => println!("Testing UMF solver\n"),
     }
 
     let mut trip = match SparseTriplet::new(5, 5, 13, Symmetry::No) {
@@ -15,22 +15,22 @@ fn test_solver(name: LinSol) {
         }
     };
 
-    trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-    trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-    trip.put(1, 0, 3.0);
-    trip.put(0, 1, 3.0);
-    trip.put(2, 1, -1.0);
-    trip.put(4, 1, 4.0);
-    trip.put(1, 2, 4.0);
-    trip.put(2, 2, -3.0);
-    trip.put(3, 2, 1.0);
-    trip.put(4, 2, 2.0);
-    trip.put(2, 3, 2.0);
-    trip.put(1, 4, 6.0);
-    trip.put(4, 4, 1.0);
+    trip.put(0, 0, 1.0).unwrap(); // << (0, 0, a00/2)
+    trip.put(0, 0, 1.0).unwrap(); // << (0, 0, a00/2)
+    trip.put(1, 0, 3.0).unwrap();
+    trip.put(0, 1, 3.0).unwrap();
+    trip.put(2, 1, -1.0).unwrap();
+    trip.put(4, 1, 4.0).unwrap();
+    trip.put(1, 2, 4.0).unwrap();
+    trip.put(2, 2, -3.0).unwrap();
+    trip.put(3, 2, 1.0).unwrap();
+    trip.put(4, 2, 2.0).unwrap();
+    trip.put(2, 3, 2.0).unwrap();
+    trip.put(1, 4, 6.0).unwrap();
+    trip.put(4, 4, 1.0).unwrap();
 
     let mut config = ConfigSolver::new();
-    config.set_solver(name);
+    config.lin_sol_kind(name);
     let mut solver = match Solver::new(config) {
         Ok(v) => v,
         Err(e) => {
@@ -86,8 +86,8 @@ fn test_solver(name: LinSol) {
         }
     };
 
-    trip_singular.put(0, 0, 1.0);
-    trip_singular.put(4, 4, 1.0);
+    trip_singular.put(0, 0, 1.0).unwrap();
+    trip_singular.put(4, 4, 1.0).unwrap();
     match solver.initialize(&trip_singular) {
         Err(e) => {
             println!("FAIL(initialize singular matrix): {}", e);
@@ -104,7 +104,7 @@ fn test_solver(name: LinSol) {
 
 fn main() {
     println!("Running Mem Check\n");
-    test_solver(LinSol::Mmp);
-    test_solver(LinSol::Umf);
+    test_solver(LinSolKind::Mmp);
+    test_solver(LinSolKind::Umf);
     println!("Done\n");
 }
