@@ -5,26 +5,26 @@
 //! # Example - solving a sparse linear system
 //!
 //! ```
-//! use russell_lab::*;
-//! use russell_sparse::*;
+//! use russell_lab::{Matrix, Vector};
+//! use russell_sparse::{ConfigSolver, Solver, SparseTriplet, Symmetry, StrError};
 //!
-//! fn main() -> Result<(), &'static str> {
+//! fn main() -> Result<(), StrError> {
 //!
 //!     // allocate a square matrix
 //!     let mut trip = SparseTriplet::new(5, 5, 13, Symmetry::No)?;
-//!     trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-//!     trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-//!     trip.put(1, 0, 3.0);
-//!     trip.put(0, 1, 3.0);
-//!     trip.put(2, 1, -1.0);
-//!     trip.put(4, 1, 4.0);
-//!     trip.put(1, 2, 4.0);
-//!     trip.put(2, 2, -3.0);
-//!     trip.put(3, 2, 1.0);
-//!     trip.put(4, 2, 2.0);
-//!     trip.put(2, 3, 2.0);
-//!     trip.put(1, 4, 6.0);
-//!     trip.put(4, 4, 1.0);
+//!     trip.put(0, 0, 1.0)?; // << (0, 0, a00/2)
+//!     trip.put(0, 0, 1.0)?; // << (0, 0, a00/2)
+//!     trip.put(1, 0, 3.0)?;
+//!     trip.put(0, 1, 3.0)?;
+//!     trip.put(2, 1, -1.0)?;
+//!     trip.put(4, 1, 4.0)?;
+//!     trip.put(1, 2, 4.0)?;
+//!     trip.put(2, 2, -3.0)?;
+//!     trip.put(3, 2, 1.0)?;
+//!     trip.put(4, 2, 2.0)?;
+//!     trip.put(2, 3, 2.0)?;
+//!     trip.put(1, 4, 6.0)?;
+//!     trip.put(4, 4, 1.0)?;
 //!
 //!     // print matrix
 //!     let (m, n) = trip.dims();
@@ -66,6 +66,9 @@ pub fn desc() -> String {
     "Sparse matrix tools and solvers".to_string()
 }
 
+/// Defines a type alias for the error type as a static string
+pub type StrError = &'static str;
+
 mod config_solver;
 mod enums;
 mod read_matrix_market;
