@@ -1,4 +1,4 @@
-use super::Symmetry;
+use crate::{StrError, Symmetry};
 use russell_lab::{Matrix, Vector};
 use russell_openblas::to_i32;
 use std::fmt;
@@ -40,7 +40,18 @@ impl SparseTriplet {
     /// * `max` -- The maximum number fo non-zero values in the sparse matrix,
     ///            including entries with repeated indices
     /// * `sym` -- Specifies how the data is stored regarding symmetry
-    pub fn new(nrow: usize, ncol: usize, max: usize, sym: Symmetry) -> Result<Self, &'static str> {
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let trip = SparseTriplet::new(3, 3, 4, Symmetry::No)?;
+    ///     Ok(())
+    /// }
+    /// ```
+    pub fn new(nrow: usize, ncol: usize, max: usize, sym: Symmetry) -> Result<Self, StrError> {
         if nrow == 0 || ncol == 0 || max == 0 {
             return Err("nrow, ncol, and max must all be greater than zero");
         }
@@ -57,19 +68,41 @@ impl SparseTriplet {
     }
 
     /// Puts the next triple (i,j,aij) into the Triplet
-    pub fn put(&mut self, i: usize, j: usize, aij: f64) {
-        assert!(i < self.nrow);
-        assert!(j < self.ncol);
-        assert!(self.pos < self.max);
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let mut trip = SparseTriplet::new(3, 3, 4, Symmetry::No)?;
+    ///     trip.put(0, 0, 1.0)?;
+    ///     trip.put(1, 1, 2.0)?;
+    ///     trip.put(2, 2, 3.0)?;
+    ///     trip.put(0, 1, 4.0)?;
+    ///     Ok(())
+    /// }
+    /// ```
+    pub fn put(&mut self, i: usize, j: usize, aij: f64) -> Result<(), StrError> {
+        if i >= self.nrow {
+            return Err("sparse matrix row index is out of bounds");
+        }
+        if j >= self.ncol {
+            return Err("sparse matrix column index is out of bounds");
+        }
+        if self.pos >= self.max {
+            return Err("current nnz (number of non-zeros) reached maximum limit");
+        }
         let i_i32 = to_i32(i);
         let j_i32 = to_i32(j);
         self.indices_i[self.pos] = i_i32;
         self.indices_j[self.pos] = j_i32;
         self.values_aij[self.pos] = aij;
         self.pos += 1;
+        Ok(())
     }
 
-    /// Returns the (nrow x ncol) dimensions of the matrix represented by this Triplet
+    /// Returns the (nrow, ncol) dimensions of the matrix represented by this Triplet
     ///
     /// # Output
     ///
@@ -79,17 +112,75 @@ impl SparseTriplet {
     /// # Example
     ///
     /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let trip = SparseTriplet::new(2, 2, 1, Symmetry::No)?;
-    /// assert_eq!(trip.dims(), (2, 2));
-    /// # Ok(())
-    /// # }
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let trip = SparseTriplet::new(2, 2, 1, Symmetry::No)?;
+    ///     assert_eq!(trip.dims(), (2, 2));
+    ///     Ok(())
+    /// }
     /// ```
     pub fn dims(&self) -> (usize, usize) {
         (self.nrow, self.ncol)
     }
 
+    /// Returns the current number of non-zero values (nnz)
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let trip = SparseTriplet::new(2, 2, 1, Symmetry::No)?;
+    ///     assert_eq!(trip.nnz_current(), 0);
+    ///     Ok(())
+    /// }
+    /// ```
+    pub fn nnz_current(&self) -> usize {
+        self.pos
+    }
+
+    /// Returns the maximum allowed number of non-zero values (nnz)
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let trip = SparseTriplet::new(2, 2, 1, Symmetry::No)?;
+    ///     assert_eq!(trip.nnz_max(), 1);
+    ///     Ok(())
+    /// }
+    /// ```
+    pub fn nnz_max(&self) -> usize {
+        self.max
+    }
+
+    /// Resets the position of the current non-zero value, allowing using "put" from scratch
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
+    ///
+    /// fn main() -> Result<(), StrError> {
+    ///     let mut trip = SparseTriplet::new(3, 3, 4, Symmetry::No)?;
+    ///     trip.put(0, 0, 1.0)?;
+    ///     trip.put(1, 1, 2.0)?;
+    ///     trip.put(2, 2, 3.0)?;
+    ///     trip.put(0, 1, 4.0)?;
+    ///     assert_eq!(trip.nnz_current(), 4);
+    ///     trip.reset();
+    ///     assert_eq!(trip.nnz_current(), 0);
+    ///     Ok(())
+    /// }
+    /// ```
+    pub fn reset(&mut self) {
+        self.pos = 0;
+    }
+
     /// Converts the triples data to a matrix, up to a limit
     ///
     /// # Input
@@ -99,46 +190,45 @@ impl SparseTriplet {
     /// # Example
     ///
     /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// // import
-    /// use russell_lab::*;
-    /// use russell_sparse::*;
+    /// use russell_lab::{Matrix};
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
     ///
-    /// // define (4 x 4) sparse matrix with 6+1 non-zero values
-    /// // (with an extra ij-repeated entry)
-    /// let mut trip = SparseTriplet::new(4, 4, 6+1, Symmetry::No)?;
-    /// trip.put(0, 0, 0.5); // (0, 0, a00/2)
-    /// trip.put(0, 0, 0.5); // (0, 0, a00/2)
-    /// trip.put(0, 1, 2.0);
-    /// trip.put(1, 0, 3.0);
-    /// trip.put(1, 1, 4.0);
-    /// trip.put(2, 2, 5.0);
-    /// trip.put(3, 3, 6.0);
+    /// fn main() -> Result<(), StrError> {
+    ///     // define (4 x 4) sparse matrix with 6+1 non-zero values
+    ///     // (with an extra ij-repeated entry)
+    ///     let mut trip = SparseTriplet::new(4, 4, 6+1, Symmetry::No)?;
+    ///     trip.put(0, 0, 0.5)?; // (0, 0, a00/2)
+    ///     trip.put(0, 0, 0.5)?; // (0, 0, a00/2)
+    ///     trip.put(0, 1, 2.0)?;
+    ///     trip.put(1, 0, 3.0)?;
+    ///     trip.put(1, 1, 4.0)?;
+    ///     trip.put(2, 2, 5.0)?;
+    ///     trip.put(3, 3, 6.0)?;
     ///
-    /// // convert the first (3 x 3) values
-    /// let mut a = Matrix::new(3, 3);
-    /// trip.to_matrix(&mut a)?;
-    /// let correct = "┌       ┐\n\
-    ///                │ 1 2 0 │\n\
-    ///                │ 3 4 0 │\n\
-    ///                │ 0 0 5 │\n\
-    ///                └       ┘";
-    /// assert_eq!(format!("{}", a), correct);
+    ///     // convert the first (3 x 3) values
+    ///     let mut a = Matrix::new(3, 3);
+    ///     trip.to_matrix(&mut a)?;
+    ///     let correct = "┌       ┐\n\
+    ///                    │ 1 2 0 │\n\
+    ///                    │ 3 4 0 │\n\
+    ///                    │ 0 0 5 │\n\
+    ///                    └       ┘";
+    ///     assert_eq!(format!("{}", a), correct);
     ///
-    /// // convert the first (4 x 4) values
-    /// let mut b = Matrix::new(4, 4);
-    /// trip.to_matrix(&mut b)?;
-    /// let correct = "┌         ┐\n\
-    ///                │ 1 2 0 0 │\n\
-    ///                │ 3 4 0 0 │\n\
-    ///                │ 0 0 5 0 │\n\
-    ///                │ 0 0 0 6 │\n\
-    ///                └         ┘";
-    /// assert_eq!(format!("{}", b), correct);
-    /// # Ok(())
-    /// # }
+    ///     // convert the first (4 x 4) values
+    ///     let mut b = Matrix::new(4, 4);
+    ///     trip.to_matrix(&mut b)?;
+    ///     let correct = "┌         ┐\n\
+    ///                    │ 1 2 0 0 │\n\
+    ///                    │ 3 4 0 0 │\n\
+    ///                    │ 0 0 5 0 │\n\
+    ///                    │ 0 0 0 6 │\n\
+    ///                    └         ┘";
+    ///     assert_eq!(format!("{}", b), correct);
+    ///     Ok(())
+    /// }
     /// ```
-    pub fn to_matrix(&self, a: &mut Matrix) -> Result<(), &'static str> {
+    pub fn to_matrix(&self, a: &mut Matrix) -> Result<(), StrError> {
         let (m, n) = a.dims();
         if m > self.nrow || n > self.ncol {
             return Err("wrong matrix dimensions");
@@ -169,48 +259,47 @@ impl SparseTriplet {
     /// # Example
     ///
     /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// // import
-    /// use russell_lab::*;
-    /// use russell_sparse::*;
+    /// use russell_lab::{Matrix, Vector};
+    /// use russell_sparse::{SparseTriplet, Symmetry, StrError};
     ///
-    /// // set sparse matrix (4 x 3) with 6 non-zeros
-    /// let mut trip = SparseTriplet::new(4, 3, 6, Symmetry::No)?;
-    /// trip.put(0, 0, 1.0);
-    /// trip.put(1, 0, 2.0);
-    /// trip.put(1, 1, 3.0);
-    /// trip.put(2, 0, 4.0);
-    /// trip.put(3, 0, 5.0);
-    /// trip.put(3, 2, 6.0);
+    /// fn main() -> Result<(), StrError> {
+    ///     // set sparse matrix (4 x 3) with 6 non-zeros
+    ///     let mut trip = SparseTriplet::new(4, 3, 6, Symmetry::No)?;
+    ///     trip.put(0, 0, 1.0)?;
+    ///     trip.put(1, 0, 2.0)?;
+    ///     trip.put(1, 1, 3.0)?;
+    ///     trip.put(2, 0, 4.0)?;
+    ///     trip.put(3, 0, 5.0)?;
+    ///     trip.put(3, 2, 6.0)?;
     ///
-    /// // check matrix
-    /// let (m, n) = trip.dims();
-    /// let mut a = Matrix::new(m, n);
-    /// trip.to_matrix(&mut a)?;
-    /// let correct_a = "┌       ┐\n\
-    ///                  │ 1 0 0 │\n\
-    ///                  │ 2 3 0 │\n\
-    ///                  │ 4 0 0 │\n\
-    ///                  │ 5 0 6 │\n\
-    ///                  └       ┘";
-    /// assert_eq!(format!("{}", a), correct_a);
+    ///     // check matrix
+    ///     let (m, n) = trip.dims();
+    ///     let mut a = Matrix::new(m, n);
+    ///     trip.to_matrix(&mut a)?;
+    ///     let correct_a = "┌       ┐\n\
+    ///                      │ 1 0 0 │\n\
+    ///                      │ 2 3 0 │\n\
+    ///                      │ 4 0 0 │\n\
+    ///                      │ 5 0 6 │\n\
+    ///                      └       ┘";
+    ///     assert_eq!(format!("{}", a), correct_a);
     ///
-    /// // perform mat-vec-mul
-    /// let u = Vector::from(&[1.0, 1.0, 1.0]);
-    /// let v = trip.mat_vec_mul(&u)?;
+    ///     // perform mat-vec-mul
+    ///     let u = Vector::from(&[1.0, 1.0, 1.0]);
+    ///     let v = trip.mat_vec_mul(&u)?;
     ///
-    /// // check vector
-    /// let correct_v = "┌    ┐\n\
-    ///                  │  1 │\n\
-    ///                  │  5 │\n\
-    ///                  │  4 │\n\
-    ///                  │ 11 │\n\
-    ///                  └    ┘";
-    /// assert_eq!(format!("{}", v), correct_v);
-    /// # Ok(())
-    /// # }
+    ///     // check vector
+    ///     let correct_v = "┌    ┐\n\
+    ///                      │  1 │\n\
+    ///                      │  5 │\n\
+    ///                      │  4 │\n\
+    ///                      │ 11 │\n\
+    ///                      └    ┘";
+    ///     assert_eq!(format!("{}", v), correct_v);
+    ///     Ok(())
+    /// }
     /// ```
-    pub fn mat_vec_mul(&self, u: &Vector) -> Result<Vector, &'static str> {
+    pub fn mat_vec_mul(&self, u: &Vector) -> Result<Vector, StrError> {
         if u.dim() != self.ncol {
             return Err("u.ndim must equal a.ncol");
         }
@@ -253,7 +342,7 @@ impl fmt::Display for SparseTriplet {
 #[cfg(test)]
 mod tests {
     use super::SparseTriplet;
-    use crate::Symmetry;
+    use crate::{StrError, Symmetry};
     use russell_chk::assert_vec_approx_eq;
     use russell_lab::{Matrix, Vector};
 
@@ -274,7 +363,7 @@ mod tests {
     }
 
     #[test]
-    fn new_works() -> Result<(), &'static str> {
+    fn new_works() -> Result<(), StrError> {
         let trip = SparseTriplet::new(3, 3, 5, Symmetry::No)?;
         assert_eq!(trip.nrow, 3);
         assert_eq!(trip.ncol, 3);
@@ -285,52 +374,57 @@ mod tests {
     }
 
     #[test]
-    #[should_panic]
-    fn put_panics_on_wrong_values_1() {
+    fn put_fails_on_wrong_values() {
         let mut trip = SparseTriplet::new(1, 1, 1, Symmetry::No).unwrap();
-        trip.put(1, 0, 0.0);
+        assert_eq!(
+            trip.put(1, 0, 0.0).err(),
+            Some("sparse matrix row index is out of bounds")
+        );
+        assert_eq!(
+            trip.put(0, 1, 0.0).err(),
+            Some("sparse matrix column index is out of bounds")
+        );
+        assert_eq!(trip.put(0, 0, 0.0).err(), None); // << will tak all spots
+        assert_eq!(
+            trip.put(0, 0, 0.0).err(),
+            Some("current nnz (number of non-zeros) reached maximum limit")
+        );
     }
 
     #[test]
-    #[should_panic]
-    fn put_panics_on_wrong_values_2() {
-        let mut trip = SparseTriplet::new(1, 1, 1, Symmetry::No).unwrap();
-        trip.put(0, 1, 0.0);
-    }
-
-    #[test]
-    #[should_panic]
-    fn put_panics_on_wrong_values_3() {
-        let mut trip = SparseTriplet::new(1, 1, 1, Symmetry::No).unwrap();
-        trip.put(0, 0, 0.0); // << all spots occupied
-        trip.put(0, 0, 0.0);
-    }
-
-    #[test]
-    fn put_works() -> Result<(), &'static str> {
+    fn put_works() -> Result<(), StrError> {
         let mut trip = SparseTriplet::new(3, 3, 5, Symmetry::No)?;
-        trip.put(0, 0, 1.0);
+        trip.put(0, 0, 1.0)?;
         assert_eq!(trip.pos, 1);
-        trip.put(0, 1, 2.0);
+        trip.put(0, 1, 2.0)?;
         assert_eq!(trip.pos, 2);
-        trip.put(1, 0, 3.0);
+        trip.put(1, 0, 3.0)?;
         assert_eq!(trip.pos, 3);
-        trip.put(1, 1, 4.0);
+        trip.put(1, 1, 4.0)?;
         assert_eq!(trip.pos, 4);
-        trip.put(2, 2, 5.0);
+        trip.put(2, 2, 5.0)?;
         assert_eq!(trip.pos, 5);
         Ok(())
     }
 
     #[test]
-    fn dims_works() -> Result<(), &'static str> {
-        let trip = SparseTriplet::new(3, 2, 1, Symmetry::No)?;
+    fn getters_and_reset_work() -> Result<(), StrError> {
+        let mut trip = SparseTriplet::new(3, 2, 4, Symmetry::No)?;
+        assert_eq!(trip.nnz_current(), 0);
+        trip.put(0, 0, 1.0)?;
+        trip.put(0, 1, 4.0)?;
+        trip.put(1, 1, 2.0)?;
+        trip.put(2, 0, 3.0)?;
         assert_eq!(trip.dims(), (3, 2));
+        assert_eq!(trip.nnz_current(), 4);
+        assert_eq!(trip.nnz_max(), 4);
+        trip.reset();
+        assert_eq!(trip.nnz_current(), 0);
         Ok(())
     }
 
     #[test]
-    fn to_matrix_fails_on_wrong_dims() -> Result<(), &'static str> {
+    fn to_matrix_fails_on_wrong_dims() -> Result<(), StrError> {
         let trip = SparseTriplet::new(1, 1, 1, Symmetry::No)?;
         let mut a_2x1 = Matrix::new(2, 1);
         let mut a_1x2 = Matrix::new(1, 2);
@@ -340,13 +434,13 @@ mod tests {
     }
 
     #[test]
-    fn to_matrix_works() -> Result<(), &'static str> {
+    fn to_matrix_works() -> Result<(), StrError> {
         let mut trip = SparseTriplet::new(3, 3, 5, Symmetry::No)?;
-        trip.put(0, 0, 1.0);
-        trip.put(0, 1, 2.0);
-        trip.put(1, 0, 3.0);
-        trip.put(1, 1, 4.0);
-        trip.put(2, 2, 5.0);
+        trip.put(0, 0, 1.0)?;
+        trip.put(0, 1, 2.0)?;
+        trip.put(1, 0, 3.0)?;
+        trip.put(1, 1, 4.0)?;
+        trip.put(2, 2, 5.0)?;
         let mut a = Matrix::new(3, 3);
         trip.to_matrix(&mut a)?;
         assert_eq!(a.get(0, 0), 1.0);
@@ -362,22 +456,22 @@ mod tests {
     }
 
     #[test]
-    fn to_matrix_with_duplicates_works() -> Result<(), &'static str> {
+    fn to_matrix_with_duplicates_works() -> Result<(), StrError> {
         // allocate a square matrix
         let mut trip = SparseTriplet::new(5, 5, 13, Symmetry::No)?;
-        trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-        trip.put(0, 0, 1.0); // << (0, 0, a00/2)
-        trip.put(1, 0, 3.0);
-        trip.put(0, 1, 3.0);
-        trip.put(2, 1, -1.0);
-        trip.put(4, 1, 4.0);
-        trip.put(1, 2, 4.0);
-        trip.put(2, 2, -3.0);
-        trip.put(3, 2, 1.0);
-        trip.put(4, 2, 2.0);
-        trip.put(2, 3, 2.0);
-        trip.put(1, 4, 6.0);
-        trip.put(4, 4, 1.0);
+        trip.put(0, 0, 1.0)?; // << (0, 0, a00/2)
+        trip.put(0, 0, 1.0)?; // << (0, 0, a00/2)
+        trip.put(1, 0, 3.0)?;
+        trip.put(0, 1, 3.0)?;
+        trip.put(2, 1, -1.0)?;
+        trip.put(4, 1, 4.0)?;
+        trip.put(1, 2, 4.0)?;
+        trip.put(2, 2, -3.0)?;
+        trip.put(3, 2, 1.0)?;
+        trip.put(4, 2, 2.0)?;
+        trip.put(2, 3, 2.0)?;
+        trip.put(1, 4, 6.0)?;
+        trip.put(4, 4, 1.0)?;
 
         // print matrix
         let (m, n) = trip.dims();
@@ -395,7 +489,7 @@ mod tests {
     }
 
     #[test]
-    fn mat_vec_mul_fails_on_wrong_input() -> Result<(), &'static str> {
+    fn mat_vec_mul_fails_on_wrong_input() -> Result<(), StrError> {
         let trip = SparseTriplet::new(2, 2, 1, Symmetry::No)?;
         let u = Vector::new(3);
         assert_eq!(trip.mat_vec_mul(&u).err(), Some("u.ndim must equal a.ncol"));
@@ -403,26 +497,26 @@ mod tests {
     }
 
     #[test]
-    fn mat_vec_mul_works() -> Result<(), &'static str> {
+    fn mat_vec_mul_works() -> Result<(), StrError> {
         //  1.0  2.0  3.0  4.0  5.0
         //  0.1  0.2  0.3  0.4  0.5
         // 10.0 20.0 30.0 40.0 50.0
         let mut trip = SparseTriplet::new(3, 5, 15, Symmetry::No)?;
-        trip.put(0, 0, 1.0);
-        trip.put(0, 1, 2.0);
-        trip.put(0, 2, 3.0);
-        trip.put(0, 3, 4.0);
-        trip.put(0, 4, 5.0);
-        trip.put(1, 0, 0.1);
-        trip.put(1, 1, 0.2);
-        trip.put(1, 2, 0.3);
-        trip.put(1, 3, 0.4);
-        trip.put(1, 4, 0.5);
-        trip.put(2, 0, 10.0);
-        trip.put(2, 1, 20.0);
-        trip.put(2, 2, 30.0);
-        trip.put(2, 3, 40.0);
-        trip.put(2, 4, 50.0);
+        trip.put(0, 0, 1.0)?;
+        trip.put(0, 1, 2.0)?;
+        trip.put(0, 2, 3.0)?;
+        trip.put(0, 3, 4.0)?;
+        trip.put(0, 4, 5.0)?;
+        trip.put(1, 0, 0.1)?;
+        trip.put(1, 1, 0.2)?;
+        trip.put(1, 2, 0.3)?;
+        trip.put(1, 3, 0.4)?;
+        trip.put(1, 4, 0.5)?;
+        trip.put(2, 0, 10.0)?;
+        trip.put(2, 1, 20.0)?;
+        trip.put(2, 2, 30.0)?;
+        trip.put(2, 3, 40.0)?;
+        trip.put(2, 4, 50.0)?;
         let u = Vector::from(&[0.1, 0.2, 0.3, 0.4, 0.5]);
         let correct_v = &[5.5, 0.55, 55.0];
         let v = trip.mat_vec_mul(&u)?;
@@ -431,32 +525,32 @@ mod tests {
     }
 
     #[test]
-    fn mat_vec_mul_sym_part_works() -> Result<(), &'static str> {
+    fn mat_vec_mul_sym_part_works() -> Result<(), StrError> {
         // 2
         // 1  2     sym
         // 1  2  9
         // 3  1  1  7
         // 2  1  5  1  8
         let mut trip = SparseTriplet::new(5, 5, 15, Symmetry::GeneralTriangular)?;
-        trip.put(0, 0, 2.0);
-        trip.put(1, 1, 2.0);
-        trip.put(2, 2, 9.0);
-        trip.put(3, 3, 7.0);
-        trip.put(4, 4, 8.0);
+        trip.put(0, 0, 2.0)?;
+        trip.put(1, 1, 2.0)?;
+        trip.put(2, 2, 9.0)?;
+        trip.put(3, 3, 7.0)?;
+        trip.put(4, 4, 8.0)?;
 
-        trip.put(1, 0, 1.0);
+        trip.put(1, 0, 1.0)?;
 
-        trip.put(2, 0, 1.0);
-        trip.put(2, 1, 2.0);
+        trip.put(2, 0, 1.0)?;
+        trip.put(2, 1, 2.0)?;
 
-        trip.put(3, 0, 3.0);
-        trip.put(3, 1, 1.0);
-        trip.put(3, 2, 1.0);
+        trip.put(3, 0, 3.0)?;
+        trip.put(3, 1, 1.0)?;
+        trip.put(3, 2, 1.0)?;
 
-        trip.put(4, 0, 2.0);
-        trip.put(4, 1, 1.0);
-        trip.put(4, 2, 5.0);
-        trip.put(4, 3, 1.0);
+        trip.put(4, 0, 2.0)?;
+        trip.put(4, 1, 1.0)?;
+        trip.put(4, 2, 5.0)?;
+        trip.put(4, 3, 1.0)?;
         let u = Vector::from(&[-629.0 / 98.0, 237.0 / 49.0, -53.0 / 49.0, 62.0 / 49.0, 23.0 / 14.0]);
         let correct_v = &[-2.0, 4.0, 3.0, -5.0, 1.0];
         let v = trip.mat_vec_mul(&u)?;
@@ -465,42 +559,42 @@ mod tests {
     }
 
     #[test]
-    fn mat_vec_mul_sym_full_works() -> Result<(), &'static str> {
+    fn mat_vec_mul_sym_full_works() -> Result<(), StrError> {
         // 2  1  1  3  2
         // 1  2  2  1  1
         // 1  2  9  1  5
         // 3  1  1  7  1
         // 2  1  5  1  8
         let mut trip = SparseTriplet::new(5, 5, 25, Symmetry::General)?;
-        trip.put(0, 0, 2.0);
-        trip.put(1, 1, 2.0);
-        trip.put(2, 2, 9.0);
-        trip.put(3, 3, 7.0);
-        trip.put(4, 4, 8.0);
+        trip.put(0, 0, 2.0)?;
+        trip.put(1, 1, 2.0)?;
+        trip.put(2, 2, 9.0)?;
+        trip.put(3, 3, 7.0)?;
+        trip.put(4, 4, 8.0)?;
 
-        trip.put(1, 0, 1.0);
-        trip.put(0, 1, 1.0);
+        trip.put(1, 0, 1.0)?;
+        trip.put(0, 1, 1.0)?;
 
-        trip.put(2, 0, 1.0);
-        trip.put(0, 2, 1.0);
-        trip.put(2, 1, 2.0);
-        trip.put(1, 2, 2.0);
+        trip.put(2, 0, 1.0)?;
+        trip.put(0, 2, 1.0)?;
+        trip.put(2, 1, 2.0)?;
+        trip.put(1, 2, 2.0)?;
 
-        trip.put(3, 0, 3.0);
-        trip.put(0, 3, 3.0);
-        trip.put(3, 1, 1.0);
-        trip.put(1, 3, 1.0);
-        trip.put(3, 2, 1.0);
-        trip.put(2, 3, 1.0);
+        trip.put(3, 0, 3.0)?;
+        trip.put(0, 3, 3.0)?;
+        trip.put(3, 1, 1.0)?;
+        trip.put(1, 3, 1.0)?;
+        trip.put(3, 2, 1.0)?;
+        trip.put(2, 3, 1.0)?;
 
-        trip.put(4, 0, 2.0);
-        trip.put(0, 4, 2.0);
-        trip.put(4, 1, 1.0);
-        trip.put(1, 4, 1.0);
-        trip.put(4, 2, 5.0);
-        trip.put(2, 4, 5.0);
-        trip.put(4, 3, 1.0);
-        trip.put(3, 4, 1.0);
+        trip.put(4, 0, 2.0)?;
+        trip.put(0, 4, 2.0)?;
+        trip.put(4, 1, 1.0)?;
+        trip.put(1, 4, 1.0)?;
+        trip.put(4, 2, 5.0)?;
+        trip.put(2, 4, 5.0)?;
+        trip.put(4, 3, 1.0)?;
+        trip.put(3, 4, 1.0)?;
         let u = Vector::from(&[-629.0 / 98.0, 237.0 / 49.0, -53.0 / 49.0, 62.0 / 49.0, 23.0 / 14.0]);
         let correct_v = &[-2.0, 4.0, 3.0, -5.0, 1.0];
         let v = trip.mat_vec_mul(&u)?;
@@ -509,16 +603,16 @@ mod tests {
     }
 
     #[test]
-    fn mat_vec_mul_pos_def_works() -> Result<(), &'static str> {
+    fn mat_vec_mul_pos_def_works() -> Result<(), StrError> {
         //  2  -1              2     ...
-        // -1   2  -1    =>   -1   2    
+        // -1   2  -1    =>   -1   2
         //     -1   2             -1   2
         let mut trip = SparseTriplet::new(3, 3, 5, Symmetry::PosDefTriangular)?;
-        trip.put(0, 0, 2.0);
-        trip.put(1, 1, 2.0);
-        trip.put(2, 2, 2.0);
-        trip.put(1, 0, -1.0);
-        trip.put(2, 1, -1.0);
+        trip.put(0, 0, 2.0)?;
+        trip.put(1, 1, 2.0)?;
+        trip.put(2, 2, 2.0)?;
+        trip.put(1, 0, -1.0)?;
+        trip.put(2, 1, -1.0)?;
         let u = Vector::from(&[5.0, 8.0, 7.0]);
         let correct_v = &[2.0, 4.0, 6.0];
         let v = trip.mat_vec_mul(&u)?;
@@ -527,7 +621,7 @@ mod tests {
     }
 
     #[test]
-    fn display_trait_works() -> Result<(), &'static str> {
+    fn display_trait_works() -> Result<(), StrError> {
         let trip = SparseTriplet::new(3, 3, 1, Symmetry::General)?;
         let correct: &str = "\x20\x20\x20\x20\"nrow\": 3,\n\
                              \x20\x20\x20\x20\"ncol\": 3,\n\
